# frozen_string_literal: true

require 'bigdecimal'

module LessonlyApi
  module Utils
    class CustomFieldConverter
      CONVERSIONS = {
        big_decimal: ->(val) { val.is_a?(BigDecimal) ? val : val && BigDecimal(val.to_s) },
        date: ->(val) { val.is_a?(Date) ? val : val && Date.parse(val) },
        date_time: ->(val) { val.is_a?(DateTime) ? val : val && DateTime.parse(val) }
      }.freeze

      def initialize(data_type)
        @data_type = data_type
      end

      def convert(value)
        if @data_type.is_a?(Array)
          convert_array(value)
        elsif CONVERSIONS.key?(@data_type)
          CONVERSIONS[@data_type].call(value)
        else
          wrap(value, @data_type)
        end
      end

      private

      def wrap(value, data_type)
        data_type_class = "LessonlyApi::ResourceType::#{data_type.to_s.camelize}".constantize
        value.is_a?(LessonlyApi::ResourceType::Base) ? value : data_type_class.new(value)
      end

      def convert_array(array)
        return [] if array.blank?

        array.map { |item| wrap(item, @data_type.first) }
      end
    end
  end
end
