module LessonlyApi
  class Webhook
    class << self
      def call(env)
        new(env).call
      end
    end

    attr_reader :env

    def initialize(env)
      @env = env
    end

    def call
      return [405, {"Content-Type" => "text/plain"}, ['Method Not Allowed']] unless webhook_handler
      return [401, {"Content-Type" => "text/plain"}, ['Unauthorized']] unless authorized?

      parse_webhook_data

      return [406, {"Content-Type" => "text/plain"}, ['Not Acceptable']] if raw_webhook_data.nil?

      webhook_handler.call(webhook_data)

      [200, {"Content-Type" => "text/plain"}, ['OK']]
    end

    private

    attr_reader :raw_webhook_data

    def authorized?
      env['HTTP_AUTHORIZATION'] == LessonlyApi.configuration.webhook_authentication
    end

    def parse_webhook_data
      @raw_webhook_data = JSON.parse(env['rack.input'].read)
    rescue JSON::ParserError
      nil
    end

    def webhook_data
      LessonlyApi::ResourceType::Webhook.new(raw_webhook_data)
    end

    def webhook_handler
      LessonlyApi.configuration.webhook_handler
    end
  end
end
